@@ -1,9 +1,9 @@
 /*
- *  $Id: textbox.c,v 1.118 2019/07/25 00:07:15 tom Exp $
+ *  $Id: textbox.c,v 1.136 2025/01/09 22:33:20 tom Exp $
  *
  *  textbox.c -- implements the text box
  *
- *  Copyright 2000-2018,2019	Thomas E.  Dickey
+ *  Copyright 2000-2024,2025	Thomas E. Dickey
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License, version 2.1
@@ -24,7 +24,7 @@
  *	Savio Lam (lam836@cs.cuhk.hk)
  */
 
-#include <dialog.h>
+#include <dlg_internals.h>
 #include <dlg_keys.h>
 
 #define PAGE_LENGTH	(height - 4)
@@ -133,7 +133,7 @@ read_high(MY_OBJ * obj, size_t size_read)
     buftab = xalloc(size_read + 1);
 
     if ((obj->fd_bytes_read = read(obj->fd, buftab, size_read)) != -1) {
-	int i = 0, j, n, tmpint;
+	int j;
 	long begin_line;
 
 	buftab[obj->fd_bytes_read] = '\0';	/* mark end of valid data */
@@ -179,22 +179,28 @@ read_high(MY_OBJ * obj, size_t size_read)
 
 	j = 0;
 	begin_line = 0;
-	while (j < obj->fd_bytes_read) {
-	    char ch;
+	if (obj->buf != NULL) {
+	    int i = 0;
 
-	    if (((ch = buftab[j++]) == TAB) && (dialog_vars.tab_correct != 0)) {
-		tmpint = (dialog_state.tab_len
-			  - ((int) ((long) i - begin_line) % dialog_state.tab_len));
-		for (n = 0; n < tmpint; n++)
-		    obj->buf[i++] = ' ';
-	    } else {
-		if (ch == '\n')
-		    begin_line = i + 1;
-		obj->buf[i++] = ch;
+	    while (j < obj->fd_bytes_read) {
+		char ch;
+
+		if (((ch = buftab[j++]) == TAB)
+		    && (dialog_vars.tab_correct != 0)) {
+		    int n;
+		    int tmpint = (dialog_state.tab_len
+				  - ((int) ((long) i - begin_line) % dialog_state.tab_len));
+		    for (n = 0; n < tmpint; n++)
+			obj->buf[i++] = ' ';
+		} else {
+		    if (ch == '\n')
+			begin_line = i + 1;
+		    obj->buf[i++] = ch;
+		}
 	    }
-	}
 
-	obj->buf[i] = '\0';	/* mark end of valid data */
+	    obj->buf[i] = '\0';	/* mark end of valid data */
+	}
 
     }
     if (obj->bytes_read == -1)
@@ -203,19 +209,21 @@ read_high(MY_OBJ * obj, size_t size_read)
 }
 
 static long
-find_first(MY_OBJ * obj, char *buffer, long length)
+find_first(const MY_OBJ * obj, const char *buffer, long length)
 {
-    long recount = obj->page_length;
     long result = 0;
 
-    while (length > 0) {
-	if (buffer[length] == '\n') {
-	    if (--recount < 0) {
-		result = length;
-		break;
+    if (buffer != NULL) {
+	long recount = obj->page_length;
+	while (length > 0) {
+	    if (buffer[length] == '\n') {
+		if (--recount < 0) {
+		    result = length;
+		    break;
+		}
 	    }
+	    --length;
 	}
-	--length;
     }
     return result;
 }
@@ -241,11 +249,11 @@ tabize(MY_OBJ * obj, long val, long *first_pos)
 	dlg_exiterr("Error reading file in tabize().");
 
     begin_line = count = 0;
-    if (first_pos != 0)
+    if (first_pos != NULL)
 	*first_pos = 0;
 
     for (i = 0; i < val; i++) {
-	if ((first_pos != 0) && (count >= val)) {
+	if ((first_pos != NULL) && (count >= val)) {
 	    *first_pos = find_first(obj, buftab, i);
 	    break;
 	}
@@ -272,30 +280,31 @@ static char *
 get_line(MY_OBJ * obj)
 {
     int i = 0;
-    long fpos;
 
     obj->end_reached = FALSE;
-    while (obj->buf[obj->in_buf] != '\n') {
-	if (obj->buf[obj->in_buf] == '\0') {	/* Either end of file or end of buffer reached */
-	    fpos = ftell_obj(obj);
+    if (obj->buf != NULL) {
+	while (obj->buf[obj->in_buf] != '\n') {
+	    if (obj->buf[obj->in_buf] == '\0') {	/* Either end of file or end of buffer reached */
+		long fpos = ftell_obj(obj);
 
-	    if (fpos < obj->file_size) {	/* Not end of file yet */
-		/* We've reached end of buffer, but not end of file yet, so
-		 * read next part of file into buffer
-		 */
-		read_high(obj, BUF_SIZE);
-		obj->in_buf = 0;
-	    } else {
-		if (!obj->end_reached)
-		    obj->end_reached = TRUE;
-		break;
+		if (fpos < obj->file_size) {	/* Not end of file yet */
+		    /* We've reached end of buffer, but not end of file yet, so
+		     * read next part of file into buffer
+		     */
+		    read_high(obj, BUF_SIZE);
+		    obj->in_buf = 0;
+		} else {
+		    if (!obj->end_reached)
+			obj->end_reached = TRUE;
+		    break;
+		}
+	    } else if (i < MAX_LEN)
+		obj->line[i++] = obj->buf[obj->in_buf++];
+	    else {
+		if (i == MAX_LEN)	/* Truncate lines longer than MAX_LEN characters */
+		    obj->line[i++] = '\0';
+		obj->in_buf++;
 	    }
-	} else if (i < MAX_LEN)
-	    obj->line[i++] = obj->buf[obj->in_buf++];
-	else {
-	    if (i == MAX_LEN)	/* Truncate lines longer than MAX_LEN characters */
-		obj->line[i++] = '\0';
-	    obj->in_buf++;
 	}
     }
     if (i <= MAX_LEN)
@@ -307,10 +316,10 @@ get_line(MY_OBJ * obj)
 }
 
 static bool
-match_string(MY_OBJ * obj, char *string)
+match_string(MY_OBJ * obj, const char *string)
 {
     char *match = get_line(obj);
-    return strstr(match, string) != 0;
+    return strstr(match, string) != NULL;
 }
 
 /*
@@ -360,7 +369,10 @@ back_lines(MY_OBJ * obj, long n)
 	    }
 	}
 	obj->in_buf--;
-	if (obj->buf[obj->in_buf] != '\n')
+	if (obj->buf == NULL
+	    || obj->in_buf < 0
+	    || obj->in_buf >= obj->bytes_read
+	    || obj->buf[obj->in_buf] != '\n')
 	    /* Something's wrong... */
 	    dlg_exiterr("Internal error in back_lines().");
     }
@@ -420,8 +432,21 @@ print_line(MY_OBJ * obj, int row, int width)
 	for (i = first; (i <= limit) && ((cols[i] - cols[first]) < width); ++i)
 	    last = i;
 
-	(void) waddch(obj->text, ' ');
-	(void) waddnstr(obj->text, line + indx[first], indx[last] - indx[first]);
+	if (dialog_state.color_mode & coloredContent) {
+	    bool save_colors = dialog_vars.colors;
+	    chtype my_attr = dialog_attr;
+	    x = 1;
+	    dialog_vars.colors = TRUE;
+	    wmove(obj->text, row, x);
+	    dlg_attrset(obj->text, my_attr);
+	    dlg_print_line(obj->text, &my_attr, line + first,
+			   obj->hscroll + first,
+			   obj->hscroll + last, &x);
+	    dialog_vars.colors = save_colors;
+	} else {
+	    (void) waddch(obj->text, ' ');
+	    (void) waddnstr(obj->text, line + indx[first], indx[last] - indx[first]);
+	}
 
 	getyx(obj->text, y, x);
 	if (y == row) {		/* Clear 'residue' of previous line */
@@ -601,6 +626,8 @@ perform_search(MY_OBJ * obj, int height, int width, int key, char *search_term)
 		ungetch(KEY_RESIZE);
 		/* FALLTHRU */
 	    }
+#else
+	    (void) result;
 #endif
 	    /* ESC pressed, or no search term, reprint page to clear box */
 	    dlg_attrset(obj->text, dialog_attr);
@@ -702,9 +729,9 @@ dialog_textbox(const char *title, const char *filename, int height, int width)
 #endif
     long fpos;
     int x, y, cur_x, cur_y;
-    int key = 0, fkey;
+    int key, fkey;
     int next = 0;
-    int i, code, passed_end;
+    int i, passed_end;
     char search_term[MAX_LEN + 1];
     MY_OBJ obj;
     WINDOW *dialog;
@@ -777,6 +804,7 @@ dialog_textbox(const char *title, const char *filename, int height, int width)
     dlg_attr_clear(obj.text, PAGE_LENGTH, PAGE_WIDTH, dialog_attr);
 
     while (result == DLG_EXIT_UNKNOWN) {
+	int code;
 
 	/*
 	 * Update the screen according to whether we shifted up/down by a line
@@ -824,8 +852,10 @@ dialog_textbox(const char *title, const char *filename, int height, int width)
 	next = 0;		/* ...but not scroll by a line */
 
 	key = dlg_mouse_wgetch(dialog, &fkey);
-	if (dlg_result_key(key, fkey, &result))
-	    break;
+	if (dlg_result_key(key, fkey, &result)) {
+	    if (!dlg_ok_button_key(result, &button, &key, &fkey))
+		break;
+	}
 
 	if (!fkey && (code = dlg_char_to_button(key, obj.buttons)) >= 0) {
 	    result = dlg_ok_buttoncode(code);
@@ -862,10 +892,10 @@ dialog_textbox(const char *title, const char *filename, int height, int width)
 				 FALSE, width);
 		break;
 	    case DLGK_ENTER:
-		if (dialog_vars.nook)
-		    result = DLG_EXIT_OK;
-		else
-		    result = dlg_exit_buttoncode(button);
+		result = dlg_enter_buttoncode(button);
+		break;
+	    case DLGK_LEAVE:
+		result = dlg_ok_buttoncode(button);
 		break;
 	    case DLGK_PAGE_FIRST:
 		if (!obj.begin_reached) {
@@ -958,9 +988,7 @@ dialog_textbox(const char *title, const char *filename, int height, int width)
 		width = old_width;
 		back_lines(&obj, obj.page_length);
 		/* repaint */
-		dlg_clear();
-		dlg_del_window(dialog);
-		dlg_mouse_free_regions();
+		_dlg_resize_cleanup(dialog);
 		goto retry;
 #endif
 	    }
@@ -979,6 +1007,7 @@ dialog_textbox(const char *title, const char *filename, int height, int width)
 	    }
 	}
     }
+    dlg_add_last_key(-1);
 
     dlg_del_window(dialog);
     free(obj.buf);
